#!/usr/bin/env python3
"""
Event-Driven Architecture for TuskLang Python SDK
=================================================
Event sourcing, event store, and event-driven patterns

This module provides event-driven architecture capabilities for the TuskLang Python SDK,
including event sourcing, event store, event-driven patterns, and distributed event handling.
"""

import asyncio
import json
import time
import threading
from typing import Any, Dict, List, Optional, Callable, Union, Tuple, Type
from dataclasses import dataclass, asdict
from datetime import datetime, timedelta
from enum import Enum
import logging
import uuid
from collections import defaultdict, deque
import aiofiles
try:
    import aioredis
    AIOREDIS_AVAILABLE = True
except ImportError:
    import redis
    AIOREDIS_AVAILABLE = False
    print("aioredis not available. Using redis instead.")
import sqlite3
import pickle
from abc import ABC, abstractmethod


class EventType(Enum):
    """Event type enumeration"""
    DOMAIN = "domain"
    INTEGRATION = "integration"
    COMMAND = "command"
    QUERY = "query"
    SYSTEM = "system"


class EventStatus(Enum):
    """Event status enumeration"""
    PENDING = "pending"
    PROCESSING = "processing"
    COMPLETED = "completed"
    FAILED = "failed"
    CANCELLED = "cancelled"


@dataclass
class Event:
    """Event structure"""
    event_id: str
    event_type: EventType
    aggregate_id: str
    aggregate_type: str
    event_name: str
    data: Dict[str, Any]
    metadata: Dict[str, Any]
    timestamp: datetime
    version: int
    status: EventStatus = EventStatus.PENDING


@dataclass
class Aggregate:
    """Aggregate structure"""
    aggregate_id: str
    aggregate_type: str
    version: int
    state: Dict[str, Any]
    events: List[Event]
    created_at: datetime
    updated_at: datetime


@dataclass
class EventStoreConfig:
    """Event store configuration structure"""
    store_id: str
    name: str
    storage_type: str
    connection_string: str
    max_events_per_aggregate: int
    enable_snapshots: bool
    snapshot_interval: int


class EventDrivenArchitecture:
    """Event-driven architecture system for TuskLang"""
    
    def __init__(self, config: Dict[str, Any] = None):
        self.config = config or {}
        self.logger = logging.getLogger('tusklang.eventdriven')
        
        # Initialize components
        self.event_stores = {}
        self.aggregates = {}
        self.event_handlers = defaultdict(list)
        self.command_handlers = {}
        self.query_handlers = {}
        self.snapshots = {}
        
        # Initialize architecture components
        self.event_store = EventStore()
        self.event_bus = EventBus()
        self.aggregate_repository = AggregateRepository()
        self.event_sourcer = EventSourcer()
        
        # Initialize architecture
        self.architecture_active = True
        self.event_queue = asyncio.Queue()
        self.command_queue = asyncio.Queue()
        
        # Start background processes
        self._start_background_processes()
    
    def _start_background_processes(self):
        """Start background event-driven processes"""
        # Event processor
        self.event_processor_thread = threading.Thread(target=self._event_processor_loop, daemon=True)
        self.event_processor_thread.start()
        
        # Command processor
        self.command_processor_thread = threading.Thread(target=self._command_processor_loop, daemon=True)
        self.command_processor_thread.start()
    
    def create_event_store(self, config: EventStoreConfig) -> bool:
        """Create a new event store"""
        try:
            self.event_stores[config.store_id] = self.event_store.create_store(config)
            self.logger.info(f"Created event store: {config.store_id}")
            return True
            
        except Exception as e:
            self.logger.error(f"Failed to create event store {config.store_id}: {e}")
            return False
    
    async def publish_event(self, event: Event) -> bool:
        """Publish event to event store"""
        try:
            # Store event
            await self.event_store.store_event(event)
            
            # Add to event queue for processing
            await self.event_queue.put(event)
            
            # Publish to event bus
            await self.event_bus.publish(event)
            
            self.logger.info(f"Published event: {event.event_id}")
            return True
            
        except Exception as e:
            self.logger.error(f"Failed to publish event {event.event_id}: {e}")
            return False
    
    async def create_aggregate(self, aggregate_type: str, aggregate_id: str, 
                              initial_state: Dict[str, Any] = None) -> str:
        """Create a new aggregate"""
        try:
            aggregate = Aggregate(
                aggregate_id=aggregate_id,
                aggregate_type=aggregate_type,
                version=0,
                state=initial_state or {},
                events=[],
                created_at=datetime.now(),
                updated_at=datetime.now()
            )
            
            self.aggregates[aggregate_id] = aggregate
            
            # Create aggregate creation event
            event = Event(
                event_id=str(uuid.uuid4()),
                event_type=EventType.DOMAIN,
                aggregate_id=aggregate_id,
                aggregate_type=aggregate_type,
                event_name="AggregateCreated",
                data=initial_state or {},
                metadata={"operation": "create"},
                timestamp=datetime.now(),
                version=1
            )
            
            await self.publish_event(event)
            
            self.logger.info(f"Created aggregate: {aggregate_id}")
            return aggregate_id
            
        except Exception as e:
            self.logger.error(f"Failed to create aggregate {aggregate_id}: {e}")
            raise
    
    async def apply_event(self, aggregate_id: str, event_name: str, 
                         data: Dict[str, Any], metadata: Dict[str, Any] = None) -> bool:
        """Apply event to aggregate"""
        if aggregate_id not in self.aggregates:
            return False
        
        try:
            aggregate = self.aggregates[aggregate_id]
            
            # Create event
            event = Event(
                event_id=str(uuid.uuid4()),
                event_type=EventType.DOMAIN,
                aggregate_id=aggregate_id,
                aggregate_type=aggregate.aggregate_type,
                event_name=event_name,
                data=data,
                metadata=metadata or {},
                timestamp=datetime.now(),
                version=aggregate.version + 1
            )
            
            # Apply event to aggregate
            aggregate.events.append(event)
            aggregate.version = event.version
            aggregate.updated_at = datetime.now()
            
            # Update aggregate state based on event
            await self._apply_event_to_state(aggregate, event)
            
            # Publish event
            await self.publish_event(event)
            
            return True
            
        except Exception as e:
            self.logger.error(f"Failed to apply event to aggregate {aggregate_id}: {e}")
            return False
    
    async def _apply_event_to_state(self, aggregate: Aggregate, event: Event):
        """Apply event to aggregate state"""
        # This is a simplified state application
        # In a real implementation, you would have specific event handlers for each event type
        
        if event.event_name == "AggregateCreated":
            aggregate.state.update(event.data)
        elif event.event_name == "StateUpdated":
            aggregate.state.update(event.data)
        elif event.event_name == "PropertyChanged":
            property_name = event.data.get("property")
            property_value = event.data.get("value")
            if property_name:
                aggregate.state[property_name] = property_value
    
    def get_aggregate(self, aggregate_id: str) -> Optional[Aggregate]:
        """Get aggregate by ID"""
        return self.aggregates.get(aggregate_id)
    
    async def get_aggregate_events(self, aggregate_id: str, 
                                 from_version: int = 0) -> List[Event]:
        """Get events for aggregate"""
        return await self.event_store.get_events(aggregate_id, from_version)
    
    def register_event_handler(self, event_name: str, handler: Callable[[Event], None]):
        """Register event handler"""
        self.event_handlers[event_name].append(handler)
        self.logger.info(f"Registered event handler for: {event_name}")
    
    def register_command_handler(self, command_name: str, handler: Callable[[Dict[str, Any]], Any]):
        """Register command handler"""
        self.command_handlers[command_name] = handler
        self.logger.info(f"Registered command handler for: {command_name}")
    
    def register_query_handler(self, query_name: str, handler: Callable[[Dict[str, Any]], Any]):
        """Register query handler"""
        self.query_handlers[query_name] = handler
        self.logger.info(f"Registered query handler for: {query_name}")
    
    async def execute_command(self, command_name: str, command_data: Dict[str, Any]) -> Any:
        """Execute command"""
        if command_name not in self.command_handlers:
            raise ValueError(f"Command handler not found: {command_name}")
        
        try:
            # Add command to queue
            command = {
                "command_name": command_name,
                "command_data": command_data,
                "timestamp": datetime.now()
            }
            
            await self.command_queue.put(command)
            
            # Execute command
            handler = self.command_handlers[command_name]
            result = await handler(command_data)
            
            return result
            
        except Exception as e:
            self.logger.error(f"Command execution error: {e}")
            raise
    
    async def execute_query(self, query_name: str, query_data: Dict[str, Any]) -> Any:
        """Execute query"""
        if query_name not in self.query_handlers:
            raise ValueError(f"Query handler not found: {query_name}")
        
        try:
            handler = self.query_handlers[query_name]
            result = await handler(query_data)
            
            return result
            
        except Exception as e:
            self.logger.error(f"Query execution error: {e}")
            raise
    
    async def create_snapshot(self, aggregate_id: str) -> bool:
        """Create snapshot for aggregate"""
        if aggregate_id not in self.aggregates:
            return False
        
        try:
            aggregate = self.aggregates[aggregate_id]
            
            snapshot = {
                "aggregate_id": aggregate_id,
                "version": aggregate.version,
                "state": aggregate.state,
                "timestamp": datetime.now()
            }
            
            self.snapshots[aggregate_id] = snapshot
            
            # Store snapshot
            await self.event_store.store_snapshot(aggregate_id, snapshot)
            
            self.logger.info(f"Created snapshot for aggregate: {aggregate_id}")
            return True
            
        except Exception as e:
            self.logger.error(f"Failed to create snapshot for aggregate {aggregate_id}: {e}")
            return False
    
    async def restore_from_snapshot(self, aggregate_id: str) -> bool:
        """Restore aggregate from snapshot"""
        try:
            snapshot = await self.event_store.get_snapshot(aggregate_id)
            if not snapshot:
                return False
            
            # Create aggregate from snapshot
            aggregate = Aggregate(
                aggregate_id=aggregate_id,
                aggregate_type=snapshot.get("aggregate_type", "Unknown"),
                version=snapshot["version"],
                state=snapshot["state"],
                events=[],
                created_at=snapshot["timestamp"],
                updated_at=datetime.now()
            )
            
            self.aggregates[aggregate_id] = aggregate
            
            # Replay events after snapshot
            events = await self.get_aggregate_events(aggregate_id, snapshot["version"] + 1)
            for event in events:
                await self._apply_event_to_state(aggregate, event)
                aggregate.events.append(event)
                aggregate.version = event.version
            
            self.logger.info(f"Restored aggregate from snapshot: {aggregate_id}")
            return True
            
        except Exception as e:
            self.logger.error(f"Failed to restore aggregate {aggregate_id}: {e}")
            return False
    
    def get_aggregate_info(self, aggregate_id: str) -> Optional[Dict[str, Any]]:
        """Get aggregate information"""
        if aggregate_id not in self.aggregates:
            return None
        
        aggregate = self.aggregates[aggregate_id]
        
        return {
            "aggregate_id": aggregate_id,
            "aggregate_type": aggregate.aggregate_type,
            "version": aggregate.version,
            "state": aggregate.state,
            "events_count": len(aggregate.events),
            "created_at": aggregate.created_at.isoformat(),
            "updated_at": aggregate.updated_at.isoformat(),
            "has_snapshot": aggregate_id in self.snapshots
        }
    
    def list_aggregates(self) -> List[Dict[str, Any]]:
        """List all aggregates"""
        return [
            self.get_aggregate_info(aggregate_id)
            for aggregate_id in self.aggregates.keys()
        ]
    
    def _event_processor_loop(self):
        """Event processing background loop"""
        while self.architecture_active:
            try:
                # Process events from queue
                loop = asyncio.new_event_loop()
                asyncio.set_event_loop(loop)
                
                async def process_events():
                    while not self.event_queue.empty():
                        event = await self.event_queue.get()
                        await self._process_event(event)
                
                loop.run_until_complete(process_events())
                loop.close()
                
                time.sleep(0.1)  # Process every 100ms
                
            except Exception as e:
                self.logger.error(f"Event processor error: {e}")
                time.sleep(1)
    
    def _command_processor_loop(self):
        """Command processing background loop"""
        while self.architecture_active:
            try:
                # Process commands from queue
                loop = asyncio.new_event_loop()
                asyncio.set_event_loop(loop)
                
                async def process_commands():
                    while not self.command_queue.empty():
                        command = await self.command_queue.get()
                        await self._process_command(command)
                
                loop.run_until_complete(process_commands())
                loop.close()
                
                time.sleep(0.1)  # Process every 100ms
                
            except Exception as e:
                self.logger.error(f"Command processor error: {e}")
                time.sleep(1)
    
    async def _process_event(self, event: Event):
        """Process a single event"""
        try:
            # Call event handlers
            if event.event_name in self.event_handlers:
                for handler in self.event_handlers[event.event_name]:
                    await handler(event)
            
            # Update event status
            event.status = EventStatus.COMPLETED
            
        except Exception as e:
            self.logger.error(f"Event processing error: {e}")
            event.status = EventStatus.FAILED
    
    async def _process_command(self, command: Dict[str, Any]):
        """Process a single command"""
        try:
            command_name = command["command_name"]
            command_data = command["command_data"]
            
            if command_name in self.command_handlers:
                handler = self.command_handlers[command_name]
                await handler(command_data)
            
        except Exception as e:
            self.logger.error(f"Command processing error: {e}")


class EventStore:
    """Event store for persistence"""
    
    def __init__(self):
        self.logger = logging.getLogger('tusklang.eventdriven.store')
        self.stores = {}
        self.connections = {}
    
    def create_store(self, config: EventStoreConfig):
        """Create event store"""
        try:
            if config.storage_type == "sqlite":
                return self._create_sqlite_store(config)
            elif config.storage_type == "redis":
                return self._create_redis_store(config)
            elif config.storage_type == "file":
                return self._create_file_store(config)
            else:
                raise ValueError(f"Unsupported storage type: {config.storage_type}")
                
        except Exception as e:
            self.logger.error(f"Failed to create event store: {e}")
            raise
    
    def _create_sqlite_store(self, config: EventStoreConfig):
        """Create SQLite event store"""
        conn = sqlite3.connect(config.connection_string)
        
        # Create tables
        conn.execute("""
            CREATE TABLE IF NOT EXISTS events (
                event_id TEXT PRIMARY KEY,
                aggregate_id TEXT,
                event_type TEXT,
                event_name TEXT,
                data TEXT,
                metadata TEXT,
                timestamp TEXT,
                version INTEGER
            )
        """)
        
        conn.execute("""
            CREATE TABLE IF NOT EXISTS snapshots (
                aggregate_id TEXT PRIMARY KEY,
                version INTEGER,
                state TEXT,
                timestamp TEXT
            )
        """)
        
        conn.commit()
        
        self.connections[config.store_id] = conn
        return config
    
    def _create_redis_store(self, config: EventStoreConfig):
        """Create Redis event store"""
        # Redis connection will be established when needed
        return config
    
    def _create_file_store(self, config: EventStoreConfig):
        """Create file-based event store"""
        # File store will be created when needed
        return config
    
    async def store_event(self, event: Event):
        """Store event"""
        try:
            # Store in memory first
            if event.aggregate_id not in self.stores:
                self.stores[event.aggregate_id] = []
            
            self.stores[event.aggregate_id].append(event)
            
            # Store in persistent storage
            await self._persist_event(event)
            
        except Exception as e:
            self.logger.error(f"Failed to store event: {e}")
            raise
    
    async def _persist_event(self, event: Event):
        """Persist event to storage"""
        # This is a simplified persistence
        # In production, you would implement proper storage logic
        pass
    
    async def get_events(self, aggregate_id: str, from_version: int = 0) -> List[Event]:
        """Get events for aggregate"""
        if aggregate_id not in self.stores:
            return []
        
        events = self.stores[aggregate_id]
        return [event for event in events if event.version > from_version]
    
    async def store_snapshot(self, aggregate_id: str, snapshot: Dict[str, Any]):
        """Store snapshot"""
        # This is a simplified snapshot storage
        # In production, you would implement proper snapshot storage
        pass
    
    async def get_snapshot(self, aggregate_id: str) -> Optional[Dict[str, Any]]:
        """Get snapshot for aggregate"""
        # This is a simplified snapshot retrieval
        # In production, you would implement proper snapshot retrieval
        return None


class EventBus:
    """Event bus for event distribution"""
    
    def __init__(self):
        self.logger = logging.getLogger('tusklang.eventdriven.bus')
        self.subscribers = defaultdict(list)
    
    async def publish(self, event: Event):
        """Publish event to subscribers"""
        try:
            subscribers = self.subscribers[event.event_name]
            
            for subscriber in subscribers:
                try:
                    await subscriber(event)
                except Exception as e:
                    self.logger.error(f"Subscriber error: {e}")
                    
        except Exception as e:
            self.logger.error(f"Event publishing error: {e}")
    
    def subscribe(self, event_name: str, subscriber: Callable[[Event], None]):
        """Subscribe to events"""
        self.subscribers[event_name].append(subscriber)
        self.logger.info(f"Subscribed to events: {event_name}")


class AggregateRepository:
    """Aggregate repository for aggregate management"""
    
    def __init__(self):
        self.logger = logging.getLogger('tusklang.eventdriven.repository')
        self.aggregates = {}
    
    def save(self, aggregate: Aggregate):
        """Save aggregate"""
        self.aggregates[aggregate.aggregate_id] = aggregate
    
    def get(self, aggregate_id: str) -> Optional[Aggregate]:
        """Get aggregate"""
        return self.aggregates.get(aggregate_id)
    
    def delete(self, aggregate_id: str):
        """Delete aggregate"""
        if aggregate_id in self.aggregates:
            del self.aggregates[aggregate_id]


class EventSourcer:
    """Event sourcing utilities"""
    
    def __init__(self):
        self.logger = logging.getLogger('tusklang.eventdriven.sourcer')
    
    def replay_events(self, aggregate_id: str, events: List[Event], 
                     state_handler: Callable[[Dict[str, Any], Event], Dict[str, Any]]) -> Dict[str, Any]:
        """Replay events to rebuild state"""
        state = {}
        
        for event in events:
            state = state_handler(state, event)
        
        return state
    
    def create_event_stream(self, events: List[Event]) -> List[Event]:
        """Create ordered event stream"""
        return sorted(events, key=lambda e: (e.timestamp, e.version))


# Global event-driven architecture instance
event_driven_architecture = EventDrivenArchitecture()


def create_event_store(store_id: str, name: str, storage_type: str, 
                      connection_string: str, max_events_per_aggregate: int = 1000,
                      enable_snapshots: bool = True, snapshot_interval: int = 100) -> bool:
    """Create a new event store"""
    config = EventStoreConfig(
        store_id=store_id,
        name=name,
        storage_type=storage_type,
        connection_string=connection_string,
        max_events_per_aggregate=max_events_per_aggregate,
        enable_snapshots=enable_snapshots,
        snapshot_interval=snapshot_interval
    )
    
    return event_driven_architecture.create_event_store(config)


async def publish_event(event_type: str, aggregate_id: str, aggregate_type: str,
                       event_name: str, data: Dict[str, Any], 
                       metadata: Dict[str, Any] = None) -> bool:
    """Publish event"""
    event_type_enum = EventType(event_type.lower())
    
    event = Event(
        event_id=str(uuid.uuid4()),
        event_type=event_type_enum,
        aggregate_id=aggregate_id,
        aggregate_type=aggregate_type,
        event_name=event_name,
        data=data,
        metadata=metadata or {},
        timestamp=datetime.now(),
        version=1
    )
    
    return await event_driven_architecture.publish_event(event)


async def create_aggregate(aggregate_type: str, aggregate_id: str, 
                          initial_state: Dict[str, Any] = None) -> str:
    """Create a new aggregate"""
    return await event_driven_architecture.create_aggregate(aggregate_type, aggregate_id, initial_state)


async def apply_event(aggregate_id: str, event_name: str, 
                     data: Dict[str, Any], metadata: Dict[str, Any] = None) -> bool:
    """Apply event to aggregate"""
    return await event_driven_architecture.apply_event(aggregate_id, event_name, data, metadata)


def get_aggregate(aggregate_id: str) -> Optional[Aggregate]:
    """Get aggregate by ID"""
    return event_driven_architecture.get_aggregate(aggregate_id)


async def get_aggregate_events(aggregate_id: str, from_version: int = 0) -> List[Event]:
    """Get events for aggregate"""
    return await event_driven_architecture.get_aggregate_events(aggregate_id, from_version)


def register_event_handler(event_name: str, handler: Callable[[Event], None]):
    """Register event handler"""
    event_driven_architecture.register_event_handler(event_name, handler)


def register_command_handler(command_name: str, handler: Callable[[Dict[str, Any]], Any]):
    """Register command handler"""
    event_driven_architecture.register_command_handler(command_name, handler)


def register_query_handler(query_name: str, handler: Callable[[Dict[str, Any]], Any]):
    """Register query handler"""
    event_driven_architecture.register_query_handler(query_name, handler)


async def execute_command(command_name: str, command_data: Dict[str, Any]) -> Any:
    """Execute command"""
    return await event_driven_architecture.execute_command(command_name, command_data)


async def execute_query(query_name: str, query_data: Dict[str, Any]) -> Any:
    """Execute query"""
    return await event_driven_architecture.execute_query(query_name, query_data)


async def create_snapshot(aggregate_id: str) -> bool:
    """Create snapshot for aggregate"""
    return await event_driven_architecture.create_snapshot(aggregate_id)


async def restore_from_snapshot(aggregate_id: str) -> bool:
    """Restore aggregate from snapshot"""
    return await event_driven_architecture.restore_from_snapshot(aggregate_id)


def get_aggregate_info(aggregate_id: str) -> Optional[Dict[str, Any]]:
    """Get aggregate information"""
    return event_driven_architecture.get_aggregate_info(aggregate_id)


def list_aggregates() -> List[Dict[str, Any]]:
    """List all aggregates"""
    return event_driven_architecture.list_aggregates()


if __name__ == "__main__":
    print("Event-Driven Architecture for TuskLang Python SDK")
    print("=" * 50)
    
    # Test event-driven architecture
    print("\n1. Testing Event Store Creation:")
    
    # Create event store
    create_event_store("main-store", "Main Event Store", "sqlite", ":memory:")
    
    # Test aggregate creation
    print("\n2. Testing Aggregate Creation:")
    
    async def test_event_driven():
        # Create aggregate
        aggregate_id = await create_aggregate("User", "user-123", {"name": "John Doe", "email": "john@example.com"})
        print(f"  Created aggregate: {aggregate_id}")
        
        # Apply events
        await apply_event(aggregate_id, "UserUpdated", {"name": "John Smith"})
        await apply_event(aggregate_id, "EmailChanged", {"email": "john.smith@example.com"})
        
        # Get aggregate
        aggregate = get_aggregate(aggregate_id)
        print(f"  Aggregate version: {aggregate.version}")
        print(f"  Aggregate state: {aggregate.state}")
        
        # Register event handler
        def user_updated_handler(event):
            print(f"  Event handler: {event.event_name} - {event.data}")
        
        register_event_handler("UserUpdated", user_updated_handler)
        
        # Register command handler
        async def update_user_command(command_data):
            aggregate_id = command_data["aggregate_id"]
            name = command_data["name"]
            await apply_event(aggregate_id, "UserUpdated", {"name": name})
            return {"success": True}
        
        register_command_handler("UpdateUser", update_user_command)
        
        # Execute command
        result = await execute_command("UpdateUser", {"aggregate_id": aggregate_id, "name": "Jane Doe"})
        print(f"  Command result: {result}")
        
        # Get aggregate info
        info = get_aggregate_info(aggregate_id)
        print(f"  Aggregate info: {info}")
        
        # List aggregates
        aggregates = list_aggregates()
        print(f"  Total aggregates: {len(aggregates)}")
    
    # Run async test
    asyncio.run(test_event_driven())
    
    print("\nEvent-driven architecture testing completed!") 